(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@vue-monaco/loader'), require('vue-demi')) :
  typeof define === 'function' && define.amd ? define(['exports', '@vue-monaco/loader', 'vue-demi'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.vue_monaco_editor = {}, global.vue_monaco_loader, global.VueDemi));
})(this, (function (exports, loader, vueDemi) { 'use strict';

  function noop() {
  }
  function processSize(size) {
    return !/^\d+$/.test(size) ? size : `${size}px`;
  }
  function getOrCreateModel(monaco, value, language, path) {
    return getModel(monaco, path) || createModel(monaco, value, language, path);
  }
  function getModel(monaco, path) {
    return monaco.editor.getModel(createModelUri(monaco, path));
  }
  function createModel(monaco, value, language, path) {
    return monaco.editor.createModel(
      value,
      language,
      path ? createModelUri(monaco, path) : void 0
    );
  }
  function createModelUri(monaco, path) {
    return monaco.Uri.parse(path);
  }
  function getSlotHelper(slots) {
    return typeof slots == "function" ? slots() : slots;
  }

  function useStyle(props, isEditorReady) {
    const getWrapperStyle = vueDemi.computed(() => {
      return {
        display: "flex",
        position: "relative",
        textAlign: "initial",
        width: processSize(props.width),
        height: processSize(props.height)
      };
    });
    const getContainerStyle = vueDemi.computed(() => {
      const style = {
        width: "100%"
      };
      if (!isEditorReady.value)
        style.display = "none";
      return style;
    });
    const getLoadingStyle = vueDemi.computed(() => {
      return {
        display: "flex",
        height: "100%",
        width: "100%",
        justifyContent: "center",
        alignItems: "center"
      };
    });
    return { getWrapperStyle, getContainerStyle, getLoadingStyle };
  }

  function useMonaco() {
    const monacoRef = vueDemi.shallowRef(loader.getMonacoInstance());
    const isLoadFailed = vueDemi.ref(false);
    let cancelable;
    vueDemi.onMounted(async () => {
      try {
        if (monacoRef.value)
          return;
        cancelable = loader.init();
        monacoRef.value = await cancelable;
      } catch (error) {
        if ((error == null ? void 0 : error.type) !== "cancelation") {
          isLoadFailed.value = true;
          console.error("Monaco initialization error:", error);
        } else {
          isLoadFailed.value = false;
        }
      }
    });
    const unload = () => cancelable == null ? void 0 : cancelable.cancel();
    return {
      monacoRef,
      unload,
      isLoadFailed
    };
  }

  function useEditor(props, ctx, key) {
    const { monacoRef, unload, isLoadFailed } = useMonaco();
    const editorRef = vueDemi.shallowRef(null);
    const containerRef = vueDemi.shallowRef(null);
    const isEditorReady = vueDemi.computed(() => !!monacoRef.value && !!editorRef.value);
    const { getWrapperStyle, getContainerStyle, getLoadingStyle } = useStyle({
      width: props.width,
      height: props.height
    }, isEditorReady);
    function whenMonacoIsReady(callback) {
      return function(...args) {
        if (isEditorReady.value)
          callback.apply(this, args);
      };
    }
    vueDemi.watch(
      () => props.theme,
      whenMonacoIsReady((newTheme) => {
        var _a;
        return (_a = monacoRef.value) == null ? void 0 : _a.editor.setTheme(newTheme);
      })
    );
    vueDemi.watch(
      () => props.options,
      (newOptions) => {
        var _a;
        return (_a = editorRef.value) == null ? void 0 : _a.updateOptions(newOptions);
      },
      { deep: true }
    );
    const render = () => {
      var _a, _b, _c;
      return vueDemi.h(
        "section",
        {
          style: getWrapperStyle.value
        },
        [
          !isEditorReady.value && vueDemi.h(
            "div",
            {
              style: getLoadingStyle.value
            },
            isLoadFailed.value ? ctx.slots.failure ? getSlotHelper((_a = ctx.slots) == null ? void 0 : _a.failure) : "load failed" : ((_b = ctx.slots) == null ? void 0 : _b.loading) ? getSlotHelper((_c = ctx.slots) == null ? void 0 : _c.loading) : "loading..."
          ),
          vueDemi.h("div", {
            ref: containerRef,
            key,
            style: getContainerStyle.value,
            class: props.className
          })
        ]
      );
    };
    return { render, monacoRef, containerRef, editorRef, unload, whenMonacoIsReady };
  }

  const diffEditorProps = {
    /**
     * The original source (left one) value
     */
    original: {
      type: String,
      default: ""
    },
    /**
     * The modified source (right one) value
     */
    modified: {
      type: String,
      default: ""
    },
    /**
     * Language for the both models - original and modified
     */
    language: {
      type: String
    },
    /**
     * This prop gives you the opportunity to specify the language of the
     * original source separately, otherwise, it will get the value of the language property
     */
    originalLanguage: {
      type: String
    },
    /**
     * This prop gives you the opportunity to specify the language of the
     * modified source separately, otherwise, it will get the value of language property
     */
    modifiedLanguage: {
      type: String
    },
    /**
     * Path for the "original" model
     * Will be passed as a third argument to `.createModel` method
     * `monaco.editor.createModel(..., ..., monaco.Uri.parse(originalModelPath))`
     */
    originalModelPath: {
      type: String
    },
    /**
     * Path for the "modified" model
     * Will be passed as a third argument to `.createModel` method
     * `monaco.editor.createModel(..., ..., monaco.Uri.parse(modifiedModelPath))`
     */
    modifiedModelPath: {
      type: String
    },
    /**
     * Indicator whether to dispose the current original model when the DiffEditor is unmounted or not
     * @default false
     */
    keepCurrentOriginalModel: {
      type: Boolean
    },
    /**
     * Indicator whether to dispose the current modified model when the DiffEditor is unmounted or not
     * @default false
     */
    keepCurrentModifiedModel: {
      type: Boolean,
      default: false
    },
    /**
     * The theme for the monaco
     * Available options "vs-dark" | "light"
     * Define new themes by `monaco.editor.defineTheme`
     * @default "light"
     */
    theme: {
      type: String,
      default: "light"
    },
    /**
     * IDiffEditorConstructionOptions
     */
    options: {
      type: Object,
      default: () => ({})
    },
    /**
     * Width of the editor wrapper
     * @default "100%"
     */
    width: {
      type: [Number, String],
      default: "100%"
    },
    /**
     * Height of the editor wrapper
     * @default "100%"
     */
    height: {
      type: [Number, String],
      default: "100%"
    },
    /**
     * Class name for the editor container
     */
    className: {
      type: String
    }
  };

  var __defProp$1 = Object.defineProperty;
  var __getOwnPropSymbols$1 = Object.getOwnPropertySymbols;
  var __hasOwnProp$1 = Object.prototype.hasOwnProperty;
  var __propIsEnum$1 = Object.prototype.propertyIsEnumerable;
  var __defNormalProp$1 = (obj, key, value) => key in obj ? __defProp$1(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
  var __spreadValues$1 = (a, b) => {
    for (var prop in b || (b = {}))
      if (__hasOwnProp$1.call(b, prop))
        __defNormalProp$1(a, prop, b[prop]);
    if (__getOwnPropSymbols$1)
      for (var prop of __getOwnPropSymbols$1(b)) {
        if (__propIsEnum$1.call(b, prop))
          __defNormalProp$1(a, prop, b[prop]);
      }
    return a;
  };
  var MonacoDiffEditor = vueDemi.defineComponent({
    name: "MonacoDiffEditor",
    props: diffEditorProps,
    slots: Object,
    setup(props, ctx) {
      const { emit } = ctx;
      const { render, monacoRef, containerRef, editorRef, unload, whenMonacoIsReady } = useEditor(props, ctx, "vue-monaco-diff-editor");
      vueDemi.watch(
        () => props.originalModelPath,
        whenMonacoIsReady(() => {
          var _a;
          const originalEditor = (_a = editorRef.value) == null ? void 0 : _a.getOriginalEditor();
          const model = getOrCreateModel(
            monacoRef.value,
            props.original || "",
            props.originalLanguage || props.language || "text",
            props.originalModelPath || ""
          );
          if (model !== (originalEditor == null ? void 0 : originalEditor.getModel()))
            originalEditor == null ? void 0 : originalEditor.setModel(model);
        })
      );
      vueDemi.watch(
        () => props.modifiedModelPath,
        whenMonacoIsReady(() => {
          const modifiedEditor = editorRef.value.getModifiedEditor();
          const model = getOrCreateModel(
            monacoRef.value,
            props.modified || "",
            props.modifiedLanguage || props.language || "text",
            props.modifiedModelPath || ""
          );
          if (model !== modifiedEditor.getModel())
            modifiedEditor.setModel(model);
        })
      );
      vueDemi.watch(
        () => props.modified,
        whenMonacoIsReady(() => {
          const modifiedEditor = editorRef.value.getModifiedEditor();
          if (modifiedEditor.getOption(monacoRef.value.editor.EditorOption.readOnly)) {
            modifiedEditor.setValue(props.modified || "");
          } else {
            if (props.modified !== modifiedEditor.getValue()) {
              modifiedEditor.executeEdits("", [
                {
                  range: modifiedEditor.getModel().getFullModelRange(),
                  text: props.modified || "",
                  forceMoveMarkers: true
                }
              ]);
              modifiedEditor.pushUndoStop();
            }
          }
        })
      );
      vueDemi.watch(
        () => props.original,
        whenMonacoIsReady(() => {
          var _a, _b;
          (_b = (_a = editorRef.value) == null ? void 0 : _a.getModel()) == null ? void 0 : _b.original.setValue((props == null ? void 0 : props.original) || "");
        })
      );
      vueDemi.watch(
        () => [props.language, props.originalLanguage, props.modifiedLanguage],
        whenMonacoIsReady(() => {
          const { original, modified } = editorRef.value.getModel();
          monacoRef.value.editor.setModelLanguage(original, props.originalLanguage || props.language || "text");
          monacoRef.value.editor.setModelLanguage(modified, props.originalLanguage || props.language || "text");
        })
      );
      const createDiffEditor = () => {
        var _a;
        if (!containerRef.value || !monacoRef.value || editorRef.value)
          return;
        emit("beforeMount", monacoRef.value);
        editorRef.value = monacoRef.value.editor.createDiffEditor(containerRef.value, __spreadValues$1({
          automaticLayout: true,
          autoIndent: "brackets",
          theme: props.theme,
          formatOnPaste: true,
          formatOnType: true
        }, props.options));
        const originalModel = getOrCreateModel(
          monacoRef.value,
          props.original || "",
          props.originalLanguage || props.language || "text",
          props.originalModelPath || ""
        );
        const modifiedModel = getOrCreateModel(
          monacoRef.value,
          props.modified || "",
          props.modifiedLanguage || props.language || "text",
          props.modifiedModelPath || ""
        );
        (_a = editorRef.value) == null ? void 0 : _a.setModel({
          original: originalModel,
          modified: modifiedModel
        });
        emit("mount", editorRef.value, monacoRef.value);
      };
      vueDemi.onMounted(() => {
        const stop = vueDemi.watch(
          monacoRef,
          () => {
            if (containerRef.value && monacoRef.value) {
              vueDemi.nextTick(() => stop());
              createDiffEditor();
            }
          },
          { immediate: true }
        );
      });
      vueDemi.onUnmounted(() => {
        if (editorRef.value)
          disposeEditor();
        else
          unload();
      });
      function disposeEditor() {
        var _a, _b, _c, _d;
        const models = (_a = editorRef.value) == null ? void 0 : _a.getModel();
        if (!props.keepCurrentOriginalModel)
          (_b = models == null ? void 0 : models.original) == null ? void 0 : _b.dispose();
        if (!props.keepCurrentModifiedModel)
          (_c = models == null ? void 0 : models.modified) == null ? void 0 : _c.dispose();
        (_d = editorRef.value) == null ? void 0 : _d.dispose();
      }
      return render;
    }
  });

  function useValidator(emit, monacoRef, editorRef) {
    const disposeValidator = vueDemi.ref(noop);
    const stop = vueDemi.watch(
      () => [monacoRef, editorRef],
      () => {
        if (monacoRef.value && editorRef.value) {
          vueDemi.nextTick(() => stop());
          const changeMarkersListener = monacoRef.value.editor.onDidChangeMarkers((uris) => {
            var _a, _b;
            const editorUri = (_b = (_a = editorRef.value) == null ? void 0 : _a.getModel()) == null ? void 0 : _b.uri;
            if (editorUri) {
              const currentEditorHasMarkerChanges = uris.find((uri) => uri.path === editorUri.path);
              if (currentEditorHasMarkerChanges) {
                const markers = monacoRef.value.editor.getModelMarkers({
                  resource: editorUri
                });
                emit("validate", markers);
              }
            }
          });
          disposeValidator.value = () => changeMarkersListener == null ? void 0 : changeMarkersListener.dispose();
        }
      }
    );
    return { disposeValidator };
  }

  const editorProps = {
    /**
     * Default value of the current model
     */
    defaultValue: {
      type: String
    },
    /**
     * Default language of the current model
     */
    defaultLanguage: {
      type: String
    },
    /**
     * Default path of the current model
     * Will be passed as the third argument to `.createModel` method
     * `monaco.editor.createModel(..., ..., monaco.Uri.parse(defaultPath))`
     */
    defaultPath: {
      type: String
    },
    /**
     * Value of the current model
     */
    value: {
      type: String,
      value: ""
    },
    /**
     * Language of the current model
     */
    language: {
      type: String,
      default: "javascript"
    },
    /**
     * Path of the current model
     * Will be passed as the third argument to `.createModel` method
     * `monaco.editor.createModel(..., ..., monaco.Uri.parse(defaultPath))`
     */
    path: {
      type: String,
      default: ""
    },
    /**
     * The theme for the monaco
     * Available options "vs-dark" | "light"
     * Define new themes by `monaco.editor.defineTheme`
     * @default "light"
     */
    theme: {
      type: String,
      default: "light"
    },
    /**
     * The line to jump on it
     */
    line: {
      type: Number
    },
    /**
     * IStandaloneEditorConstructionOptions
     */
    options: {
      type: Object,
      default: () => ({})
    },
    /**
     * IEditorOverrideServices
     */
    overrideServices: {
      type: Object
    },
    /**
     * Indicator whether to save the models' view states between model changes or not
     * Defaults to true
     */
    saveViewState: {
      type: Boolean,
      default: true
    },
    /**
     * Indicator whether to dispose the current model when the Editor is unmounted or not
     * @default false
     */
    keepCurrentModel: {
      type: Boolean,
      default: false
    },
    /**
     * Width of the editor wrapper
     * @default "100%"
     */
    width: {
      type: [Number, String],
      default: "100%"
    },
    /**
     * Height of the editor wrapper
     * @default "100%"
     */
    height: {
      type: [Number, String],
      default: "100%"
    },
    /**
     * Class name for the editor container
     */
    className: {
      type: String
    }
  };

  var __defProp = Object.defineProperty;
  var __getOwnPropSymbols = Object.getOwnPropertySymbols;
  var __hasOwnProp = Object.prototype.hasOwnProperty;
  var __propIsEnum = Object.prototype.propertyIsEnumerable;
  var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
  var __spreadValues = (a, b) => {
    for (var prop in b || (b = {}))
      if (__hasOwnProp.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    if (__getOwnPropSymbols)
      for (var prop of __getOwnPropSymbols(b)) {
        if (__propIsEnum.call(b, prop))
          __defNormalProp(a, prop, b[prop]);
      }
    return a;
  };
  const viewStates = /* @__PURE__ */ new Map();
  var MonacoEditor = vueDemi.defineComponent({
    name: "MonacoEditor",
    props: editorProps,
    emits: ["update:value", "beforeMount", "mount", "change", "validate"],
    slots: Object,
    setup(props, ctx) {
      const { emit } = ctx;
      const {
        render,
        containerRef,
        editorRef,
        monacoRef,
        unload,
        whenMonacoIsReady
      } = useEditor(props, ctx, "vue-monaco-editor");
      const { disposeValidator } = useValidator(emit, monacoRef, editorRef);
      const preventTriggerChangeEvent = vueDemi.ref(false);
      vueDemi.watch(
        () => props.path,
        whenMonacoIsReady((newPath, oldPath) => {
          var _a, _b, _c;
          const model = getOrCreateModel(
            monacoRef.value,
            props.defaultValue || props.value || "",
            props.defaultLanguage || props.language || "",
            props.path || props.defaultPath || ""
          );
          if (model !== ((_a = editorRef.value) == null ? void 0 : _a.getModel())) {
            if (props.saveViewState)
              viewStates.set(oldPath, editorRef.value.saveViewState());
            (_b = editorRef.value) == null ? void 0 : _b.setModel(model);
            if (props.saveViewState)
              (_c = editorRef.value) == null ? void 0 : _c.restoreViewState(viewStates.get(newPath));
          }
        })
      );
      vueDemi.watch(
        () => props.value,
        whenMonacoIsReady((newValue) => {
          if (!editorRef.value || newValue === void 0)
            return;
          if (editorRef.value.getOption(monacoRef.value.editor.EditorOption.readOnly)) {
            editorRef.value.setValue(newValue);
          } else if (newValue !== editorRef.value.getValue()) {
            preventTriggerChangeEvent.value = true;
            editorRef.value.executeEdits("", [
              {
                range: editorRef.value.getModel().getFullModelRange(),
                text: newValue,
                forceMoveMarkers: true
              }
            ]);
            editorRef.value.pushUndoStop();
            preventTriggerChangeEvent.value = false;
          }
        })
      );
      vueDemi.watch(
        () => props.language,
        whenMonacoIsReady((newLang) => {
          var _a, _b;
          const model = (_a = editorRef.value) == null ? void 0 : _a.getModel();
          if (model && newLang)
            (_b = monacoRef.value) == null ? void 0 : _b.editor.setModelLanguage(model, newLang);
        })
      );
      vueDemi.watch(
        () => props.line,
        whenMonacoIsReady((newLine) => {
          var _a;
          if (newLine !== void 0)
            (_a = editorRef.value) == null ? void 0 : _a.revealLine(newLine);
        })
      );
      const createEditor = () => {
        var _a, _b, _c;
        if (!containerRef.value || !monacoRef.value || editorRef.value)
          return;
        emit("beforeMount", monacoRef.value);
        const autoCreatedModelPath = props.path || props.defaultPath;
        const defaultModel = getOrCreateModel(
          monacoRef.value,
          props.value || props.defaultValue || "",
          props.language || props.defaultLanguage || "",
          autoCreatedModelPath || ""
        );
        editorRef.value = (_a = monacoRef.value) == null ? void 0 : _a.editor.create(
          containerRef.value,
          __spreadValues({
            model: defaultModel,
            theme: props.theme,
            automaticLayout: true,
            autoIndent: "brackets",
            formatOnPaste: true,
            formatOnType: true
          }, props.options),
          props.overrideServices
        );
        (_b = editorRef.value) == null ? void 0 : _b.onDidChangeModelContent((event) => {
          if (!preventTriggerChangeEvent.value) {
            const value = editorRef.value.getValue();
            emit("update:value", value);
            emit("change", value, event);
          }
        });
        if (props.saveViewState) {
          const currentViewState = viewStates.get(autoCreatedModelPath);
          if (currentViewState)
            editorRef.value.restoreViewState(currentViewState);
        }
        if (props.line !== void 0)
          (_c = editorRef.value) == null ? void 0 : _c.revealLine(props.line);
        emit("mount", editorRef.value, monacoRef.value);
      };
      vueDemi.onMounted(() => {
        const stop = vueDemi.watch(
          () => monacoRef.value,
          (newMonacoRef) => {
            if (containerRef.value && newMonacoRef) {
              vueDemi.nextTick(() => stop());
              createEditor();
            }
          },
          { immediate: true }
        );
      });
      vueDemi.onUnmounted(() => {
        var _a;
        (_a = disposeValidator.value) == null ? void 0 : _a.call(disposeValidator);
        if (editorRef.value)
          disposeEditor();
        else
          unload();
      });
      function disposeEditor() {
        var _a, _b;
        if (props.keepCurrentModel) {
          if (props.saveViewState)
            viewStates.set(props.path, editorRef.value.saveViewState());
          else
            (_a = editorRef.value.getModel()) == null ? void 0 : _a.dispose();
        }
        (_b = editorRef.value) == null ? void 0 : _b.dispose();
      }
      return render;
    }
  });

  function install(app, options) {
    if (options)
      loader.config(options);
    app.component(MonacoEditor.name, MonacoEditor);
    app.component(MonacoDiffEditor.name, MonacoDiffEditor);
  }

  exports.loader = loader;
  exports.MonacoDiffEditor = MonacoDiffEditor;
  exports.MonacoEditor = MonacoEditor;
  exports.install = install;
  exports.useMonaco = useMonaco;

}));
